@@ -1,9 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
- *
- * Copyright (c) 2020 Marco Trevisan <marco.trevisan@canonical.com>
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2020 Marco Trevisan <marco.trevisan@canonical.com>
 
 #pragma once
 
@@ -17,8 +14,11 @@
 #include <girepository.h>
 #include <glib-object.h>  // for GType
 #include <glib.h>         // for gboolean
+#include <js/TypeDecls.h>  // for HandleValue
 
+#include "gi/js-value-inl.h"
 #include "gi/utils-inl.h"
+#include "gjs/macros.h"
 
 // GIArgument accessor templates
 //
@@ -35,6 +35,8 @@
 // gjs_arg_set(GIArgument*, T) - sets the appropriate union member for type T.
 // gjs_arg_unset<T>(GIArgument*) - sets the appropriate zero value in the
 //   appropriate union member for type T.
+// gjs_arg_steal<T>(GIArgument*) - sets the appropriate zero value in the
+//   appropriate union member for type T and returns the replaced value.
 
 template <typename T>
 [[nodiscard]] inline decltype(auto) gjs_arg_member(GIArgument* arg,
@@ -163,17 +165,24 @@ inline void gjs_arg_unset(GIArgument* arg) {
         gjs_arg_set<T, TAG>(arg, static_cast<T>(0));
 }
 
+template <typename T, GITypeTag TAG = GI_TYPE_TAG_VOID>
+[[nodiscard]] inline T gjs_arg_steal(GIArgument* arg) {
+    auto val = gjs_arg_get<T, TAG>(arg);
+    gjs_arg_unset<T, TAG>(arg);
+    return val;
+}
+
 // Implementation to store rounded (u)int64_t numbers into double
 
 template <typename BigT>
 [[nodiscard]] inline constexpr BigT max_safe_big_number() {
-    return BigT(1) << std::numeric_limits<double>::digits;
+    return (BigT(1) << std::numeric_limits<double>::digits) - 1;
 }
 
 template <typename BigT>
 [[nodiscard]] inline constexpr BigT min_safe_big_number() {
     if constexpr (std::is_signed_v<BigT>)
-        return -(max_safe_big_number<BigT>()) + 1;
+        return -(max_safe_big_number<BigT>());
 
     return std::numeric_limits<BigT>::lowest();
 }
@@ -196,3 +205,23 @@ gjs_arg_get_maybe_rounded(GIArgument* arg) {
 
     return static_cast<double>(val);
 }
+
+template <typename T>
+GJS_JSAPI_RETURN_CONVENTION inline bool gjs_arg_set_from_js_value(
+    JSContext* cx, const JS::HandleValue& value, GArgument* arg,
+    bool* out_of_range) {
+    if constexpr (Gjs::type_has_js_getter<T>())
+        return Gjs::js_value_to_c(cx, value, &gjs_arg_member<T>(arg));
+
+    Gjs::JsValueHolder::Relaxed<T> val;
+
+    if (!Gjs::js_value_to_c_checked<T>(cx, value, &val, out_of_range))
+        return false;
+
+    if (*out_of_range)
+        return false;
+
+    gjs_arg_set<T>(arg, val);
+
+    return true;
+}
