@@ -58,31 +58,34 @@ static bool jsobj_set_gproperty(JSContext* cx, JS::HandleObject object,
     if (pspec->flags & G_PARAM_CONSTRUCT_ONLY) {
         unsigned flags = GJS_MODULE_PROP_FLAGS | JSPROP_READONLY;
         GjsAutoChar camel_name = gjs_hyphen_to_camel(pspec->name);
-        JS::Rooted<JS::PropertyDescriptor> jsprop(cx);
 
-        // Ensure to call any associated setter method
-        if (!g_str_equal(underscore_name.get(), pspec->name)) {
-            if (!JS_GetPropertyDescriptor(cx, object, underscore_name, &jsprop))
+        if (g_param_spec_get_qdata(pspec, ObjectBase::custom_property_quark())) {
+            JS::Rooted<JS::PropertyDescriptor> jsprop(cx);
+
+            // Ensure to call any associated setter method
+            if (!g_str_equal(underscore_name.get(), pspec->name)) {
+                if (!JS_GetPropertyDescriptor(cx, object, underscore_name, &jsprop))
+                    return false;
+                if (jsprop.setter() &&
+                    !JS_SetProperty(cx, object, underscore_name, jsvalue))
+                    return false;
+            }
+
+            if (!g_str_equal(camel_name.get(), pspec->name)) {
+                if (!JS_GetPropertyDescriptor(cx, object, camel_name, &jsprop))
+                    return false;
+                if (jsprop.setter() &&
+                    !JS_SetProperty(cx, object, camel_name, jsvalue))
+                    return false;
+            }
+
+            if (!JS_GetPropertyDescriptor(cx, object, pspec->name, &jsprop))
                 return false;
             if (jsprop.setter() &&
-                !JS_SetProperty(cx, object, underscore_name, jsvalue))
+                !JS_SetProperty(cx, object, pspec->name, jsvalue))
                 return false;
         }
 
-        if (!g_str_equal(camel_name.get(), pspec->name)) {
-            if (!JS_GetPropertyDescriptor(cx, object, camel_name, &jsprop))
-                return false;
-            if (jsprop.setter() &&
-                !JS_SetProperty(cx, object, camel_name, jsvalue))
-                return false;
-        }
-
-        if (!JS_GetPropertyDescriptor(cx, object, pspec->name, &jsprop))
-            return false;
-        if (jsprop.setter() &&
-            !JS_SetProperty(cx, object, pspec->name, jsvalue))
-            return false;
-
         return JS_DefineProperty(cx, object, underscore_name, jsvalue, flags) &&
                JS_DefineProperty(cx, object, camel_name, jsvalue, flags) &&
                JS_DefineProperty(cx, object, pspec->name, jsvalue, flags);
