@@ -1,21 +1,22 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 // SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
 // SPDX-FileCopyrightText: 2017 Endless Mobile, Inc.
+// SPDX-FileCopyrightText: 2021 Canonical Ltd.
 // SPDX-FileContributor: Authored by: Philip Chimento <philip@endlessm.com>
 // SPDX-FileContributor: Philip Chimento <philip.chimento@gmail.com>
+// SPDX-FileContributor: Marco Trevisan <marco.trevisan@canonical.com>
 
 #ifndef GI_TOGGLE_H_
 #define GI_TOGGLE_H_
 
+#include <glib.h>  // for gboolean
+
 #include <atomic>
 #include <deque>
-#include <mutex>
+#include <thread>
 #include <utility>  // for pair
 
-#include <glib-object.h>
-#include <glib.h>
-
-#include "util/log.h"
+class ObjectInstance;
 
 /* Thread-safe queue for enqueueing toggle-up or toggle-down events on GObjects
  * from any thread. For more information, see object.cpp, comments near
@@ -27,46 +28,59 @@ public:
         UP
     };
 
-    typedef void (*Handler)(GObject *, Direction);
+    using Handler = void (*)(ObjectInstance*, Direction);
 
-private:
+ private:
     struct Item {
-        GObject *gobj;
+        Item() {}
+        Item(ObjectInstance* o, Direction d) : object(o), direction(d) {}
+        ObjectInstance* object;
         ToggleQueue::Direction direction;
-        unsigned needs_unref : 1;
     };
 
-    mutable std::mutex lock;
+    struct Locked {
+        explicit Locked(ToggleQueue* queue) { queue->lock(); }
+        ~Locked() { get_default_unlocked().maybe_unlock(); }
+        ToggleQueue* operator->() { return &get_default_unlocked(); }
+    };
+
     std::deque<Item> q;
     std::atomic_bool m_shutdown = ATOMIC_VAR_INIT(false);
 
     unsigned m_idle_id;
     Handler m_toggle_handler;
+    std::atomic<std::thread::id> m_holder = std::thread::id();
+    unsigned m_holder_ref_count = 0;
 
-    /* No-op unless GJS_VERBOSE_ENABLE_LIFECYCLE is defined to 1. */
-    inline void debug(const char* did GJS_USED_VERBOSE_LIFECYCLE,
-                      const void* what GJS_USED_VERBOSE_LIFECYCLE) {
-        gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "ToggleQueue %s %p", did, what);
+    void lock();
+    void maybe_unlock();
+    [[nodiscard]] bool is_locked() const {
+        return m_holder != std::thread::id();
+    }
+    [[nodiscard]] bool owns_lock() const {
+        return m_holder == std::this_thread::get_id();
     }
 
     [[nodiscard]] std::deque<Item>::iterator find_operation_locked(
-        const GObject* gobj, Direction direction);
+        const ObjectInstance* obj, Direction direction);
 
     [[nodiscard]] std::deque<Item>::const_iterator find_operation_locked(
-        const GObject* gobj, Direction direction) const;
-
-    [[nodiscard]] bool find_and_erase_operation_locked(const GObject* gobj,
-                                                       Direction direction);
+        const ObjectInstance* obj, Direction direction) const;
 
     static gboolean idle_handle_toggle(void *data);
     static void idle_destroy_notify(void *data);
 
+    [[nodiscard]] static ToggleQueue& get_default_unlocked() {
+        static ToggleQueue the_singleton;
+        return the_singleton;
+    }
+
  public:
     /* These two functions return a pair DOWN, UP signifying whether toggles
      * are / were queued. is_queued() just checks and does not modify. */
-    [[nodiscard]] std::pair<bool, bool> is_queued(GObject* gobj) const;
+    [[nodiscard]] std::pair<bool, bool> is_queued(ObjectInstance* obj) const;
     /* Cancels pending toggles and returns whether any were queued. */
-    std::pair<bool, bool> cancel(GObject* gobj);
+    std::pair<bool, bool> cancel(ObjectInstance* obj);
 
     /* Pops a toggle from the queue and processes it. Call this if you don't
      * want to wait for it to be processed in idle time. Returns false if queue
@@ -79,13 +93,10 @@ private:
     void shutdown(void);
 
     /* Queues a toggle to be processed in idle time. */
-    void enqueue(GObject  *gobj,
-                 Direction direction,
-                 Handler   handler);
+    void enqueue(ObjectInstance* obj, Direction direction, Handler handler);
 
-    [[nodiscard]] static ToggleQueue& get_default() {
-        static ToggleQueue the_singleton;
-        return the_singleton;
+    [[nodiscard]] static Locked get_default() {
+        return Locked(&get_default_unlocked());
     }
 };
 
