@@ -1,29 +1,9 @@
-/* profiler.cpp
- *
- * Copyright (C) 2016 Christian Hergert <christian@hergert.me>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2016 Christian Hergert <christian@hergert.me>
 
 #include <config.h>  // for ENABLE_PROFILER, HAVE_SYS_SYSCALL_H, HAVE_UNISTD_H
 
-#ifndef HAVE_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 #    include <signal.h>  // for siginfo_t, sigevent, sigaction, SIGPROF, ...
 #endif
 
@@ -33,14 +13,12 @@
 #ifdef ENABLE_PROFILER
 #    include <alloca.h>
 #    include <errno.h>
-#    include <stddef.h>  // for size_t
 #    include <stdint.h>
 #    include <stdio.h>      // for sscanf
 #    include <string.h>     // for memcpy, strlen
-#    include <sys/time.h>   // for CLOCK_MONOTONIC
 #    include <sys/types.h>  // for timer_t
 #    include <syscall.h>    // for __NR_gettid
-#    include <time.h>       // for itimerspec, timer_delete, ...
+#    include <time.h>       // for size_t, CLOCK_MONOTONIC, itimerspec, ...
 #    ifdef HAVE_UNISTD_H
 #        include <unistd.h>  // for getpid, syscall
 #    endif
@@ -51,6 +29,8 @@
 #endif
 
 #include <js/ProfilingStack.h>  // for EnableContextProfilingStack, ...
+#include <js/TypeDecls.h>
+#include <mozilla/Atomics.h>  // for ProfilingStack operators
 
 #include "gjs/context.h"
 #include "gjs/jsapi-util.h"
@@ -105,6 +85,8 @@ struct _GjsProfiler {
     /* Buffers and writes our sampled stacks */
     SysprofCaptureWriter* capture;
     GSource* periodic_flush;
+
+    SysprofCaptureWriter* target_capture;
 #endif  /* ENABLE_PROFILER */
 
     /* The filename to write to */
@@ -259,6 +241,7 @@ _gjs_profiler_free(GjsProfiler *self)
 #ifdef ENABLE_PROFILER
     g_clear_pointer(&self->capture, sysprof_capture_writer_unref);
     g_clear_pointer(&self->periodic_flush, g_source_destroy);
+    g_clear_pointer(&self->target_capture, sysprof_capture_writer_unref);
 
     if (self->fd != -1)
         close(self->fd);
@@ -424,12 +407,14 @@ gjs_profiler_start(GjsProfiler *self)
 
     g_return_if_fail(!self->capture);
 
-    struct sigaction sa = { 0 };
-    struct sigevent sev = { 0 };
-    struct itimerspec its = { 0 };
+    struct sigaction sa = {{0}};
+    struct sigevent sev = {{0}};
+    struct itimerspec its = {{0}};
     struct itimerspec old_its;
 
-    if (self->fd != -1) {
+    if (self->target_capture) {
+        self->capture = sysprof_capture_writer_ref(self->target_capture);
+    } else if (self->fd != -1) {
         self->capture = sysprof_capture_writer_new_from_fd(self->fd, 0);
         self->fd = -1;
     } else {
@@ -558,7 +543,7 @@ gjs_profiler_stop(GjsProfiler *self)
 
 #ifdef ENABLE_PROFILER
 
-    struct itimerspec its = { 0 };
+    struct itimerspec its = {{0}};
     timer_settime(self->timer, 0, &its, nullptr);
     timer_delete(self->timer);
 
@@ -672,6 +657,30 @@ gjs_profiler_chain_signal(GjsContext *context,
     return false;
 }
 
+/**
+ * gjs_profiler_set_capture_writer:
+ * @self: A #GjsProfiler
+ * @capture: (nullable): A #SysprofCaptureWriter
+ *
+ * Set the capture writer to which profiling data is written when the @self
+ * is stopped.
+ */
+void gjs_profiler_set_capture_writer(GjsProfiler* self, gpointer capture) {
+    g_return_if_fail(self);
+    g_return_if_fail(!self->running);
+
+#ifdef ENABLE_PROFILER
+    g_clear_pointer(&self->target_capture, sysprof_capture_writer_unref);
+    self->target_capture =
+        capture ? sysprof_capture_writer_ref(
+                      reinterpret_cast<SysprofCaptureWriter*>(capture))
+                : NULL;
+#else
+    // Unused in the no-profiler case
+    (void)capture;
+#endif
+}
+
 /**
  * gjs_profiler_set_filename:
  * @self: A #GjsProfiler
