@@ -7,32 +7,33 @@
 #include <cairo.h>
 
 #include <js/CallArgs.h>
-#include <js/Class.h>
+#include <js/GCPolicyAPI.h>
 #include <js/PropertyDescriptor.h>  // for JSPROP_READONLY
 #include <js/PropertySpec.h>
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
+#include <jsapi.h>    // for JS_NewObjectWithGivenProto
+#include <jspubtd.h>  // for JSProtoKey
 
-#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/macros.h"
 #include "modules/cairo-private.h"
 
-GJS_DEFINE_PROTO_ABSTRACT_WITH_PARENT("Gradient", cairo_gradient,
-                                      cairo_pattern,
-                                      JSCLASS_BACKGROUND_FINALIZE)
+// Avoid static_assert in MSVC builds
+namespace JS {
+template <>
+struct GCPolicy<void*> : public IgnoreGCPolicy<void*> {};
+}
 
-static void
-gjs_cairo_gradient_finalize(JSFreeOp *fop,
-                            JSObject *obj)
-{
-    gjs_cairo_pattern_finalize_pattern(fop, obj);
+JSObject* CairoGradient::new_proto(JSContext* cx, JSProtoKey) {
+    JS::RootedObject parent_proto(cx, CairoPattern::prototype(cx));
+    return JS_NewObjectWithGivenProto(cx, nullptr, parent_proto);
 }
 
 /* Properties */
 // clang-format off
-JSPropertySpec gjs_cairo_gradient_proto_props[] = {
+const JSPropertySpec CairoGradient::proto_props[] = {
     JS_STRING_SYM_PS(toStringTag, "Gradient", JSPROP_READONLY),
     JS_PS_END};
 // clang-format on
@@ -55,7 +56,7 @@ addColorStopRGB_func(JSContext *context,
                              "blue", &blue))
         return false;
 
-    cairo_pattern_t* pattern = gjs_cairo_pattern_get_pattern(context, obj);
+    cairo_pattern_t* pattern = CairoPattern::for_js(context, obj);
     if (!pattern)
         return false;
 
@@ -85,7 +86,7 @@ addColorStopRGBA_func(JSContext *context,
                              "alpha", &alpha))
         return false;
 
-    cairo_pattern_t* pattern = gjs_cairo_pattern_get_pattern(context, obj);
+    cairo_pattern_t* pattern = CairoPattern::for_js(context, obj);
     if (!pattern)
         return false;
 
@@ -98,11 +99,9 @@ addColorStopRGBA_func(JSContext *context,
     return true;
 }
 
-JSFunctionSpec gjs_cairo_gradient_proto_funcs[] = {
+const JSFunctionSpec CairoGradient::proto_funcs[] = {
     JS_FN("addColorStopRGB", addColorStopRGB_func, 0, 0),
     JS_FN("addColorStopRGBA", addColorStopRGBA_func, 0, 0),
     // getColorStopRGB
     // getColorStopRGBA
     JS_FS_END};
-
-JSFunctionSpec gjs_cairo_gradient_static_funcs[] = { JS_FS_END };
