@@ -5,6 +5,16 @@
 
 #include <config.h>  // for HAVE_READLINE_READLINE_H
 
+#ifdef HAVE_SIGNAL_H
+#    include <setjmp.h>
+#    include <signal.h>
+#    ifdef _WIN32
+#        define sigjmp_buf jmp_buf
+#        define siglongjmp(e, v) longjmp (e, v)
+#        define sigsetjmp(v, m) setjmp (v)
+#    endif
+#endif
+
 #ifdef HAVE_READLINE_READLINE_H
 #    include <stdio.h>  // include before readline/readline.h
 
@@ -80,6 +90,43 @@ public:
     }
 };
 
+#ifdef HAVE_SIGNAL_H
+
+// Adapted from https://stackoverflow.com/a/17035073/172999
+class AutoCatchCtrlC {
+    void (*m_prev_handler)(int);
+
+    static void handler(int signal) {
+        if (signal == SIGINT)
+            siglongjmp(jump_buffer, 1);
+    }
+
+ public:
+    static sigjmp_buf jump_buffer;
+
+    AutoCatchCtrlC() {
+        m_prev_handler = signal(SIGINT, &AutoCatchCtrlC::handler);
+    }
+
+    ~AutoCatchCtrlC() {
+        if (m_prev_handler != SIG_ERR)
+            signal(SIGINT, m_prev_handler);
+    }
+
+    void raise_default() {
+        if (m_prev_handler != SIG_ERR)
+            signal(SIGINT, m_prev_handler);
+        raise(SIGINT);
+    }
+};
+sigjmp_buf AutoCatchCtrlC::jump_buffer;
+
+#else  // !HAVE_SIGNAL_H
+
+struct AutoCatchCtrlC {};
+
+#endif  // !HAVE_SIGNAL_H
+
 [[nodiscard]] static bool gjs_console_readline(char** bufp,
                                                const char* prompt) {
 #ifdef HAVE_READLINE_READLINE_H
@@ -139,15 +186,24 @@ gjs_console_interact(JSContext *context,
                      JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp(argc, vp);
-    bool eof = false;
+    bool eof, exit_warning;
     JS::RootedObject global(context, gjs_get_import_global(context));
-    char *temp_buf = NULL;
+    char* temp_buf;
     int lineno;
     int startline;
 
+#ifndef HAVE_READLINE_READLINE_H
+    int rl_end = 0;  // nonzero if using readline and any text is typed in
+#endif
+
     JS::SetWarningReporter(context, gjs_console_warning_reporter);
 
-        /* It's an interactive filehandle; drop into read-eval-print loop. */
+    AutoCatchCtrlC ctrl_c;
+
+    // Separate initialization from declaration because of possible overwriting
+    // when siglongjmp() jumps into this function
+    eof = exit_warning = false;
+    temp_buf = nullptr;
     lineno = 1;
     do {
         /*
@@ -159,6 +215,29 @@ gjs_console_interact(JSContext *context,
         startline = lineno;
         std::string buffer;
         do {
+#ifdef HAVE_SIGNAL_H
+            // sigsetjmp() returns 0 if control flow encounters it normally, and
+            // nonzero if it's been jumped to. In the latter case, use a while
+            // loop so that we call sigsetjmp() a second time to reinit the jump
+            // buffer.
+            while (sigsetjmp(AutoCatchCtrlC::jump_buffer, 1) != 0) {
+                g_fprintf(stdout, "\n");
+                if (buffer.empty() && rl_end == 0) {
+                    if (!exit_warning) {
+                        g_fprintf(stdout,
+                                  "(To exit, press Ctrl+C again or Ctrl+D)\n");
+                        exit_warning = true;
+                    } else {
+                        ctrl_c.raise_default();
+                    }
+                } else {
+                    exit_warning = false;
+                }
+                buffer.clear();
+                startline = lineno = 1;
+            }
+#endif  // HAVE_SIGNAL_H
+
             if (!gjs_console_readline(
                     &temp_buf, startline == lineno ? "gjs> " : ".... ")) {
                 eof = true;
@@ -176,6 +255,7 @@ gjs_console_interact(JSContext *context,
             AutoReportException are(context);
             ok = gjs_console_eval_and_print(context, buffer, startline);
         }
+        exit_warning = false;
 
         GjsContextPrivate* gjs = GjsContextPrivate::from_cx(context);
         ok = gjs->run_jobs_fallible() && ok;
